//+------------------------------------------------------------------+
//|                                               IncADXWOnArray.mqh |
//|                                                          Integer |
//|                          https://login.mql5.com/ru/users/Integer |
//+------------------------------------------------------------------+
#property copyright "Integer"
#property link      "https://login.mql5.com/ru/users/Integer"
#property version   "1.00"
/*
    :
   input int            ADXWPeriod      =  14;
   input ENUM_MA_METHOD ADXWMethod      =  MODE_SMMA;

   :
   #include <IncOnArray/IncADXWOnArray.mqh>
   CADXWOnArray adx;
   
    OnInit:
   adx.Init(ADXWPeriod,ADXWMethod);

    OnCalculate:
   adx.Solve(rates_total,prev_calculated,DataClose,DataHigh,DataLow,PBuffer,MBuffer,TRBuffer,ATRBuffer,PBuffer,MBuffer,PDIBuffer,MDIBuffer,ADXRBuffer,ADXBuffer);
*/

#include <IncOnArray/IncMAOnArray.mqh>
//+------------------------------------------------------------------+
//| CADXWOnArray                                                     |
//+------------------------------------------------------------------+
class CADXWOnArray
  {
private:
   int               m_Period;
   ENUM_MA_METHOD    m_Method;
   string            m_Name;
   string            m_PDIName;
   string            m_MDIName;

   CMAOnArray        m_pma;
   CMAOnArray        m_mma;
   CMAOnArray        m_ama;
   CMAOnArray        m_atr;

   int               m_bra;
   int               m_brpm;
   void SolvePD(int i,
                double  &aDataClose[],
                double  &aDataHigh[],
                double  &aDataLow[],
                double  &aP[],
                double  &aM[],
                double  &aTR[])
     {
      double m_h=aDataHigh[i];
      double m_ph=aDataHigh[i-1];
      double m_l=aDataLow[i];
      double m_pl=aDataLow[i-1];
      double m_pc=aDataClose[i-1];
      double m_pd=m_h-m_ph;
      double m_md=m_pl-m_l;
      if(m_pd<0.0)
        {
         m_pd=0.0;
        }
      if(m_md<0.0)
        {
         m_md=0.0;
        }
      if(m_pd>m_md)
        {
         m_md=0.0;
        }
      else
        {
         if(m_pd<m_md)
           {
            m_pd=0.0;
           }
         else
           {
            m_pd=0.0;
            m_md=0.0;
           }
        }
      aTR[i]=MathMax(MathMax(MathAbs(m_h-m_l),MathAbs(m_h-m_pc)),MathAbs(m_l-m_pc));
      aP[i]=100.0*m_pd;
      aM[i]=100.0*m_md;
     }

public:
   void Init(int aPeriod=14,ENUM_MA_METHOD aMethod=MODE_EMA)
     {
      m_Period=aPeriod;
      m_Method=aMethod;
      m_ama.Init(m_Period,m_Method);
      m_pma.Init(m_Period,m_Method);
      m_mma.Init(m_Period,m_Method);
      m_atr.Init(m_Period,m_Method);
      m_Name="("+IntegerToString(m_Period)+","+m_pma.NameMethod()+")";
      m_PDIName="PDI"+m_Name;
      m_MDIName="MDI"+m_Name;
      m_Name="ADXW"+m_Name;
      m_bra=m_pma.BarsRequired()*2;
      m_brpm=m_pma.BarsRequired()+1;
     }
   void Solve(const int aRatesTotal,
              const int aPrevCalc,
              double  &aDataClose[],
              double  &aDataHigh[],
              double  &aDataLow[],
              double  &aP[],
              double  &aM[],
              double  &aTR[],
              double  &aATR[],
              double  &aPS[],
              double  &aMS[],
              double  &aPDI[],
              double  &aMDI[],
              double  &aADXR[],
              double  &aADX[]
              )
     {

      int Start=0;
      if(aPrevCalc==0)
        {
         for(int i=0;i<aRatesTotal;i++)
           {
            if(aDataClose[i]!=0 && aDataClose[i]!=EMPTY_VALUE)
              {
               Start=i+1;
               break;
              }
           }
         SolvePD(Start,aDataClose,aDataHigh,aDataLow,aP,aM,aTR);
         if(aP[Start]==0)aP[Start]=0.00000001;
         if(aM[Start]==0)aM[Start]=0.00000001;
         if(aTR[Start]==0)aTR[Start]=0.00000001;
         Start++;
        }
      else
        {
         Start=aPrevCalc-1;
        }
      for(int i=Start;i<aRatesTotal;i++)
        {
         SolvePD(i,aDataClose,aDataHigh,aDataLow,aP,aM,aTR);
        }
      m_pma.Solve(aRatesTotal,aPrevCalc,aP,aPS);
      m_mma.Solve(aRatesTotal,aPrevCalc,aM,aMS);
      m_atr.Solve(aRatesTotal,aPrevCalc,aTR,aATR);
      for(int i=Start;i<aRatesTotal;i++)
        {
         if(aATR[i]!=0)
           {
            aPDI[i]=aPS[i]/aATR[i];
            aMDI[i]=aMS[i]/aATR[i];
           }
         else
           {
            aPDI[i]=0;
            aMDI[i]=0;
           }
         double m_sum=aPDI[i]+aMDI[i];
         if(m_sum!=0.0)
           {
            aADXR[i]=100.0*MathAbs((aPDI[i]-aMDI[i])/m_sum);
           }
         else
           {
            aADXR[i]=0.0;
           }
        }
      m_ama.Solve(aRatesTotal,aPrevCalc,aADXR,aADX);
     }
   int BarsRequiredADX()
     {
      return(m_bra);
     }
   int BarsRequiredPDIMDI()
     {
      return(m_brpm);
     }
   string Name()
     {
      return(m_Name);
     }
   string PDIName()
     {
      return(m_PDIName);
     }
   string MDIName()
     {
      return(m_MDIName);
     }
   string About()
     {
      return("Integer's RADXWOnArray class. https://login.mql5.com/ru/users/Integer");
     }
  };
//+------------------------------------------------------------------+
